#!/usr/bin/env node

'use strict';

const chalk = require('chalk');
const commander = require('commander');
const pkg = require('./package.json');
const ConfigStore = require('configstore');
const co = require('co');
const prompt = require('co-prompt');
const childProcess = require('child_process');

const config = new ConfigStore(pkg.name);

function addNewProgram(path, shortcut, name) {
    let exists = (config.get(shortcut) || '')

    if (exists) {
        console.log(chalk.bold.red(`The shortcut ${shortcut} is already registered!`));
        return;
    }

    let newProgram = {
        shortcut: shortcut,
        name: name || '',
        path: path
    };

    config.set(shortcut, JSON.stringify(newProgram));
    console.log(chalk.bold.green('Program registered with sucess!'));
};

function listPrograms() {
    let registeredPrograms = config.all;

    if (Object.keys(registeredPrograms).length > 0)
        console.log(chalk.green.bold('\nRegistered programs:'));

    Object.keys(registeredPrograms).forEach((key) => {
        let program = JSON.parse(registeredPrograms[key]);

        console.log(`\n ${program.name ? `Name: ${program.name}\n ` : ''}Shortcut: ${program.shortcut}\n Path: ${program.path}`)
    });
}

function removeProgram(shortcut) {
    let programData = (config.get(shortcut) || '');

    if (!programData) {
        console.log(chalk.bold.red('Couldn\'t find the specified shortcut.'));
        return;
    }

    co(function* () {
        let shouldRemove = yield prompt.confirm('Are you sure? (Y/N) ');

        if (!shouldRemove) {
            process.exit(0);
        }

        config.del(shortcut);

        console.log(chalk.bold.green('Program removed!'));
        process.exit(0);
    });
}

function openProgram(shortcut) {
    let programData = (config.get(shortcut) || '');

    if (!programData) {
        console.log(chalk.bold.red(`Couldn't find the specified shortcut.`));
        return;
    }

    var program = JSON.parse(programData);

    console.log(chalk.bold.green('Starting program...'));

    childProcess.exec(`start "${program.shortcut}" "${program.path}"`, (error) => {
        if (error)
            console.log(chalk.bold.red(error));
        else
            console.log(chalk.bold.green('Program started!'));

        process.exit(0);
    });
}

commander.version('1.0.5');
commander.command('add <path> <shortcut> [name]').description('Add a new program.').action(addNewProgram);
commander.command('list').description('List all registered programs').action(listPrograms);
commander.command('remove <shortcut>').description('Remove a registered program').action(removeProgram);
commander.command('open <shortcut>').description('Open a registered program').action(openProgram);

commander.on('--help', function () {
    console.log(`  Examples
                        
    op add "chrome.exe --incognito" gci Google Chrome Incognito Mode
                     
    op list
                     
    op open gc
    
    op remove gci`);
});

commander.parse(process.argv);